@@ -45,13 +45,18 @@ class Adapter(object):
     :type logger: logging.Logger
     :param dict allow: Extra filters to pass when discovering API versions.
                        (optional)
+    :param dict additional_headers: Additional headers that should be attached
+                                    to every request passing through the
+                                    adapter. Headers of the same name specified
+                                    per request will take priority.
     """
 
     @positional()
     def __init__(self, session, service_type=None, service_name=None,
                  interface=None, region_name=None, endpoint_override=None,
                  version=None, auth=None, user_agent=None,
-                 connect_retries=None, logger=None, allow={}):
+                 connect_retries=None, logger=None, allow={},
+                 additional_headers=None):
         # NOTE(jamielennox): when adding new parameters to adapter please also
         # add them to the adapter call in httpclient.HTTPClient.__init__ as
         # well as to load_adapter_from_argparse below if the argument is
@@ -69,6 +74,7 @@ class Adapter(object):
         self.connect_retries = connect_retries
         self.logger = logger
         self.allow = allow
+        self.additional_headers = additional_headers or {}
 
     def _set_endpoint_filter_kwargs(self, kwargs):
         if self.service_type:
@@ -100,6 +106,9 @@ class Adapter(object):
         if self.allow:
             kwargs.setdefault('allow', self.allow)
 
+        for k, v in self.additional_headers.items():
+            kwargs.setdefault('headers', {}).setdefault(k, v)
+
         return self.session.request(url, method, **kwargs)
 
     def get_token(self, auth=None):
