@@ -47,7 +47,10 @@ class Password(loading.BaseV3Loader):
         _add_common_identity_options(options)
 
         options.extend([
-            loading.Opt('password', secret=True, help="User's password"),
+            loading.Opt('password',
+                        secret=True,
+                        prompt='Password: ',
+                        help="User's password"),
         ])
 
         return options
@@ -78,6 +81,15 @@ class Token(loading.BaseV3Loader):
 
 class _OpenIDConnectBase(loading.BaseFederationLoader):
 
+    def load_from_options(self, **kwargs):
+        if not (kwargs.get('access_token_endpoint') or
+                kwargs.get('discovery_endpoint')):
+            m = ("You have to specify either an 'access-token-endpoint' or "
+                 "a 'discovery-endpoint'.")
+            raise exceptions.OptionError(m)
+
+        return super(_OpenIDConnectBase, self).load_from_options(**kwargs)
+
     def get_options(self):
         options = super(_OpenIDConnectBase, self).get_options()
 
@@ -85,8 +97,24 @@ class _OpenIDConnectBase(loading.BaseFederationLoader):
             loading.Opt('client-id', help='OAuth 2.0 Client ID'),
             loading.Opt('client-secret', secret=True,
                         help='OAuth 2.0 Client Secret'),
+            loading.Opt('openid-scope', default="openid profile",
+                        dest="scope",
+                        help='OpenID Connect scope that is requested from '
+                             'authorization server. Note that the OpenID '
+                             'Connect specification states that "openid" '
+                             'must be always specified.'),
             loading.Opt('access-token-endpoint',
-                        help='OpenID Connect Provider Token Endpoint'),
+                        help='OpenID Connect Provider Token Endpoint. Note '
+                        'that if a discovery document is being passed this '
+                        'option will override the endpoint provided by the '
+                        'server in the discovery document.'),
+            loading.Opt('discovery-endpoint',
+                        help='OpenID Connect Discovery Document URL. '
+                        'The discovery document will be used to obtain the '
+                        'values of the access token endpoint and the '
+                        'authentication endpoint. This URL should look like '
+                        'https://idp.example.org/.well-known/'
+                        'openid-configuration'),
             loading.Opt('access-token-type',
                         help='OAuth 2.0 Authorization Server Introspection '
                              'token type, it is used to decide which type '
@@ -98,6 +126,18 @@ class _OpenIDConnectBase(loading.BaseFederationLoader):
         return options
 
 
+class OpenIDConnectClientCredentials(_OpenIDConnectBase):
+
+    @property
+    def plugin_class(self):
+        return identity.V3OidcClientCredentials
+
+    def get_options(self):
+        options = super(OpenIDConnectClientCredentials, self).get_options()
+
+        return options
+
+
 class OpenIDConnectPassword(_OpenIDConnectBase):
 
     @property
@@ -110,9 +150,6 @@ class OpenIDConnectPassword(_OpenIDConnectBase):
         options.extend([
             loading.Opt('username', help='Username'),
             loading.Opt('password', secret=True, help='Password'),
-            loading.Opt('openid-scope', default="profile",
-                        dest='scope',
-                        help='OpenID Connect scope that is requested from OP')
         ])
 
         return options
@@ -172,3 +209,46 @@ class TOTP(loading.BaseV3Loader):
         _assert_identity_options(kwargs)
 
         return super(TOTP, self).load_from_options(**kwargs)
+
+
+class TokenlessAuth(loading.BaseLoader):
+
+    @property
+    def plugin_class(self):
+        return identity.V3TokenlessAuth
+
+    def get_options(self):
+        options = super(TokenlessAuth, self).get_options()
+
+        options.extend([
+            loading.Opt('auth-url', required=True,
+                        help='Authentication URL'),
+            loading.Opt('domain-id', help='Domain ID to scope to'),
+            loading.Opt('domain-name', help='Domain name to scope to'),
+            loading.Opt('project-id', help='Project ID to scope to'),
+            loading.Opt('project-name', help='Project name to scope to'),
+            loading.Opt('project-domain-id',
+                        help='Domain ID containing project'),
+            loading.Opt('project-domain-name',
+                        help='Domain name containing project'),
+        ])
+
+        return options
+
+    def load_from_options(self, **kwargs):
+        if (not kwargs.get('domain_id') and
+                not kwargs.get('domain_name') and
+                not kwargs.get('project_id') and
+                not kwargs.get('project_name') or
+                (kwargs.get('project_name') and
+                    not (kwargs.get('project_domain_name') or
+                         kwargs.get('project_domain_id')))):
+            m = ('You need to provide either a domain_name, domain_id, '
+                 'project_id or project_name. '
+                 'If you have provided a project_name, in the V3 identity '
+                 'API a project_name is only unique within a domain so '
+                 'you must also provide either a project_domain_id or '
+                 'project_domain_name.')
+            raise exceptions.OptionError(m)
+
+        return super(TokenlessAuth, self).load_from_options(**kwargs)
